<?php

namespace  FiloBlu\AddressFieldsLengthValidation\Block\Plugin\Checkout;

use Magento\Customer\Model\Address\AbstractAddress;

class LayoutProcessor {

    public $_helper;
    public $_customerHelper;
    public $_checkoutHelper;

    public function __construct(
        \FiloBlu\AddressFieldsLengthValidation\Helper\Data $helper,
        \Magento\Customer\Helper\Address $customerHelper,
        \Magento\Checkout\Helper\Data $checkoutHelper
    ) {
        $this->_helper = $helper;
        $this->_customerHelper = $customerHelper;
        $this->_checkoutHelper = $checkoutHelper;
    }

    public function aroundProcess($subject, \Closure $proceed, $jsLayout) {

        $jsLayout = $proceed($jsLayout);

        if ($this->_helper->isEnabled()) {

            /*
            * First part is for the shipping and the second one for the billing address
            */
            // layouts
            $checkoutShippingLayout = $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'];

            $checkoutBillingLayout = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];

            $checkoutBillingLayoutAfter = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'];

            // variables
            $input_elemTmpl = 'FiloBlu_AddressFieldsLengthValidation/form/element/input';

            // error messages for the validation
            $genericErrorMessage = 'The field must be less or equal than %1 characters.';
            $firstnameLastnameErrorMessage = 'The field must be less or equal than %1 characters and the sum of firstname and lastname fields must be less or equal than %2 characters.';
            $streetErrorMessage = 'All address fields must contain a maximum of %1 characters.';
            $companyStreetErrorMessage = 'All address fields (including company) must contain a maximum of %1 characters.';

            if ($firstnameLastnameSumLength = $this->_helper->getFirstnameLastnameSumLength()) {

                $firstnameLength = $this->_helper->getFirstnameLength();
                $lastnameLength = $this->_helper->getLastnameLength();
                // FIRSTNAME: change template, add autocomplete property and set new validation rules.
                $checkoutShippingLayout['firstname']['config']['elementTmpl'] = $input_elemTmpl;

                $checkoutShippingLayout['firstname']['config']['autocompleteValue'] = $this->_helper->getFirstnameAutocomplete() ? 'on' : 'nope';
                $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/test.log');
                $logger = new \Zend\Log\Logger();
                $logger->addWriter($writer);
                $logger->info(__($firstnameLastnameErrorMessage, $firstnameLength, $firstnameLastnameSumLength));
                $checkoutShippingLayout['firstname']['validation'] = [
                    'required-entry' => true,
                    'validate-filoblu-firstname-lastname-sum-length' => [
                        'target' => 'div[name="shippingAddress.lastname"] input[name="lastname"]',
                        'length' => $firstnameLength,
                        'sumLength'=> $firstnameLastnameSumLength,
                        'message' => __($firstnameLastnameErrorMessage, $firstnameLength, $firstnameLastnameSumLength)
                    ]
                ];

                $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

                // LASTNAME: change template, add autocomplete property and set new validation rules.
                $checkoutShippingLayout['lastname']['config']['elementTmpl'] = $input_elemTmpl;

                $checkoutShippingLayout['lastname']['config']['autocompleteValue'] = $this->_helper->getLastnameAutocomplete() ? 'on' : 'nope';

                $checkoutShippingLayout['lastname']['validation'] = [
                    'required-entry' => true,
                    'validate-filoblu-firstname-lastname-sum-length' => [
                        'target' => 'div[name="shippingAddress.firstname"] input[name="firstname"]',
                        'length' => $lastnameLength,
                        'sumLength'=> $firstnameLastnameSumLength,
                        'message' => __($firstnameLastnameErrorMessage, $lastnameLength, $firstnameLastnameSumLength)
                    ]
                ];

                $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

                if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                    $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                    foreach ($configuration as $paymentGroup => $groupConfig) {
                        if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['firstname'])) {
                                // FIRSTNAME: change template, add autocomplete property and set new validation rules.
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['firstname']['config']['elementTmpl'] = $input_elemTmpl;
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['firstname']['config']['autocompleteValue'] = $this->_helper->getFirstnameAutocomplete() ? 'on' : 'nope';
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['firstname']['validation'] = [
                                    'required-entry' => true,
                                    'validate-filoblu-firstname-lastname-sum-length' => [
                                        'target' => 'div[name="billingAddressshared.lastname"] input[name="lastname"]',
                                        'length' => $firstnameLength,
                                        'sumLength'=> $firstnameLastnameSumLength,
                                        'message' => __($firstnameLastnameErrorMessage, $firstnameLength, $firstnameLastnameSumLength)
                                    ]
                                ];
                                $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                            }
                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['lastname'])) {
                                // LASTNAME: change template, add autocomplete property and set new validation rules.
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['lastname']['config']['elementTmpl'] = $input_elemTmpl;
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['lastname']['config']['autocompleteValue'] = $this->_helper->getLastnameAutocomplete() ? 'on' : 'nope';
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['lastname']['validation'] = [
                                    'required-entry' => true,
                                    'validate-filoblu-firstname-lastname-sum-length' => [
                                        'target' => 'div[name="billingAddressshared.firstname"] input[name="firstname"]',
                                        'length' => $lastnameLength,
                                        'sumLength'=> $firstnameLastnameSumLength,
                                        'message' => __($firstnameLastnameErrorMessage, $lastnameLength, $firstnameLastnameSumLength)
                                    ]
                                ];
                                $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                            }
                         }
                     }
                } else {
                    if (isset($checkoutBillingLayoutAfter['firstname'])) {

                        // FIRSTNAME: change template, add autocomplete property and set new validation rules.
                        $checkoutBillingLayoutAfter['firstname']['config']['elementTmpl'] = $input_elemTmpl;
                        $checkoutBillingLayoutAfter['firstname']['config']['autocompleteValue'] = $this->_helper->getFirstnameAutocomplete() ? 'on' : 'nope';
                        $checkoutBillingLayoutAfter['firstname']['validation'] = [
                            'required-entry' => true,
                            'validate-filoblu-firstname-lastname-sum-length' => [
                                'target' => 'div[name="billingAddressshared.lastname"] input[name="lastname"]',
                                'length' => $firstnameLength,
                                'sumLength'=> $firstnameLastnameSumLength,
                                'message' => __($firstnameLastnameErrorMessage, $firstnameLength, $firstnameLastnameSumLength)
                            ]
                        ];

                        $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;

                    }
                    if (isset($checkoutBillingLayoutAfter['lastname'])) {

                        // LASTNAME: change template, add autocomplete property and set new validation rules.
                        $checkoutBillingLayoutAfter['lastname']['config']['elementTmpl'] = $input_elemTmpl;
                        $checkoutBillingLayoutAfter['lastname']['config']['autocompleteValue'] = $this->_helper->getLastnameAutocomplete() ? 'on' : 'nope';
                        $checkoutBillingLayoutAfter['lastname']['validation'] = [
                            'required-entry' => true,
                            'validate-filoblu-firstname-lastname-sum-length' => [
                                'target' => 'div[name="billingAddressshared.firstname"] input[name="firstname"]',
                                'length' => $lastnameLength,
                                'sumLength'=> $firstnameLastnameSumLength,
                                'message' => __($firstnameLastnameErrorMessage, $lastnameLength, $firstnameLastnameSumLength)
                            ]
                        ];
                        $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                    }
                 }
             }

             if ($companyLength = $this->_helper->getCompanyLength()) {

                // COMPANY: change template, add autocomplete property and set new validation rules.
                $checkoutShippingLayout['company']['config']['elementTmpl'] = $input_elemTmpl;

                $checkoutShippingLayout['company']['config']['autocompleteValue'] = $this->_helper->getCompanyAutocomplete() ? 'on' : 'nope';

                if ($this->_helper->getCompanyStreetSum()) {
                    $streetSumLength = $this->_helper->getStreetLength();
                    $checkoutShippingLayout['company']['validation'] = [
                        'validate-filoblu-company-street-sum-length' => [
                            'target' => 'div[name="shippingAddress.company"] input[name="company"]',
                            'prefix' => 'shippingAddress',
                            'length'=> $streetSumLength,
                            'message' => __($companyStreetErrorMessage, $streetSumLength)
                        ]
                    ];
                }
                else {
                    $checkoutShippingLayout['company']['validation'] = [
                        'validate-filoblu-field-length' => [
                            'target' => 'div[name="shippingAddress.company"] input[name="company"]',
                            'length'=> $companyLength,
                            'message' => __($genericErrorMessage, $companyLength)
                        ]
                    ];
                }

                $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

                if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                    // COMPANY: change template, add autocomplete property and set new validation rules.
                    $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                    foreach ($configuration as $paymentGroup => $groupConfig) {
                        if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['company'])) {

                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['company']['config']['elementTmpl'] = $input_elemTmpl;
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['company']['config']['autocompleteValue'] = $this->_helper->getCompanyAutocomplete() ? 'on' : 'nope';

                                if ($this->_helper->getCompanyStreetSum()) {
                                    $streetSumLength = $this->_helper->getStreetLength();
                                    $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['company']['validation'] = [
                                        'validate-filoblu-company-street-sum-length' => [
                                            'target' => 'div[name="billingAddressshared.company"] input[name="company"]',
                                            'prefix' => 'billingAddressshared',
                                            'length'=> $streetSumLength,
                                            'message' => __($companyStreetErrorMessage, $streetSumLength)
                                        ]
                                    ];
                                }
                                else {
                                    $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['company']['validation'] = [
                                            'validate-filoblu-field-length' => [
                                                'target' => 'div[name="billingAddressshared.company"] input[name="company"]',
                                                'length'=> $companyLength,
                                                'message' => __($genericErrorMessage, $companyLength)
                                            ]
                                        ];
                                }

                                $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                            }
                        }
                    }
                } else {
                    if (isset($checkoutBillingLayoutAfter['company'])) {

                        // COMPANY: change template, add autocomplete property and set new validation rules.
                        $checkoutBillingLayoutAfter['company']['config']['elementTmpl'] = $input_elemTmpl;
                        $checkoutBillingLayoutAfter['company']['config']['autocompleteValue'] = $this->_helper->getCompanyAutocomplete() ? 'on' : 'nope';

                        if ($this->_helper->getCompanyStreetSum()) {
                            $streetSumLength = $this->_helper->getStreetLength();
                            $checkoutBillingLayoutAfter['company']['validation'] = [
                                'validate-filoblu-company-street-sum-length' => [
                                    'target' => 'div[name="billingAddressshared.company"] input[name="company"]',
                                    'prefix' => 'billingAddressshared',
                                    'length'=> $streetSumLength,
                                    'message' => __($companyStreetErrorMessage, $streetSumLength)
                                ]
                            ];
                        }
                        else {
                            $checkoutBillingLayoutAfter['company']['validation'] = [
                                    'validate-filoblu-field-length' => [
                                        'target' => 'div[name="billingAddressshared.company"] input[name="company"]',
                                        'length'=> $companyLength,
                                        'message' => __($genericErrorMessage, $companyLength)
                                    ]
                                ];
                        }

                        $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                    }
                }
            }

            if ($streetSumLength = $this->_helper->getStreetLength()) {

                // STREET: change template, add autocomplete property and set new validation rules.
                for ($_i = 0, $_n = $this->_customerHelper->getStreetLines(); $_i <= $_n; $_i++) {

                    $checkoutShippingLayout['street']['children'][$_i]['config']['elementTmpl'] = $input_elemTmpl;
                    $checkoutShippingLayout['street']['children'][$_i]['config']['autocompleteValue'] = $this->_helper->getStreetAutocomplete() ? 'on' : 'nope';

                    if ($this->_helper->getCompanyStreetSum()) {
                        $_validationRule = [
                            'validate-filoblu-company-street-sum-length' => [
                                'target' => 'div[name="shippingAddress.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                'prefix' => 'shippingAddress',
                                'length'=> $streetSumLength,
                                'message' => __($companyStreetErrorMessage, $streetSumLength)
                            ]
                         ];
                    }
                    else {
                        $_validationRule = [
                            'validate-filoblu-street-sum-length' => [
                                'target' => 'div[name="shippingAddress.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                'prefix' => 'shippingAddress',
                                'length'=> $streetSumLength,
                                'message' => __($streetErrorMessage, $streetSumLength)
                            ]
                         ];
                    }

                     if ($_i == 0) {
                         $_validationRule["required-entry"] = true;
                     }

                    $checkoutShippingLayout['street']['children'][$_i]['validation'] = $_validationRule;

                    $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;
                }

                if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                    // STREET: change template, add autocomplete property and set new validation rules.
                    $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                    foreach ($configuration as $paymentGroup => $groupConfig) {
                        if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['street'])) {

                                for ($_i = 0, $_n = $this->_customerHelper->getStreetLines(); $_i <= $_n; $_i++) {

                                    $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['street']['children'][$_i]['config']['elementTmpl'] = $input_elemTmpl;
                                    $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['street']['children'][$_i]['config']['autocompleteValue'] = $this->_helper->getStreetAutocomplete() ? 'on' : 'nope';

                                    if ($this->_helper->getCompanyStreetSum()) {
                                        $_validationRule = [
                                            'validate-filoblu-company-street-sum-length' => [
                                                'target' => 'div[name="billingAddressshared.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                                'prefix' => 'billingAddressshared',
                                                'length'=> $streetSumLength,
                                                'message' => __($companyStreetErrorMessage, $streetSumLength)
                                            ]
                                         ];
                                    }
                                    else {
                                        $_validationRule = [
                                            'validate-filoblu-street-sum-length' => [
                                                'target' => 'div[name="billingAddressshared.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                                'prefix' => 'billingAddressshared',
                                                'length'=> $streetSumLength,
                                                'message' => __($streetErrorMessage, $streetSumLength)
                                            ]
                                         ];
                                    }

                                     if ($_i == 0) {
                                         $_validationRule["required-entry"] = true;
                                     }

                                     $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['street']['children'][$_i]['validation'] = $_validationRule;

                                     $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                                }
                            }
                        }
                    }
                } else {
                    if (isset($checkoutBillingLayoutAfter['street'])) {

                        // STREET: change template, add autocomplete property and set new validation rules.
                        for ($_i = 0, $_n = $this->_customerHelper->getStreetLines(); $_i <= $_n; $_i++) {

                            $checkoutBillingLayoutAfter['street']['children'][$_i]['config']['elementTmpl'] = $input_elemTmpl;
                            $checkoutBillingLayoutAfter['street']['children'][$_i]['config']['autocompleteValue'] = $this->_helper->getStreetAutocomplete() ? 'on' : 'nope';

                            if ($this->_helper->getCompanyStreetSum()) {
                                $_validationRule = [
                                    'validate-filoblu-company-street-sum-length' => [
                                        'target' => 'div[name="billingAddressshared.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                        'prefix' => 'billingAddressshared',
                                        'length'=> $streetSumLength,
                                        'message' => __($companyStreetErrorMessage, $streetSumLength)
                                    ]
                                 ];
                            }
                            else {
                                $_validationRule = [
                                    'validate-filoblu-street-sum-length' => [
                                        'target' => 'div[name="billingAddressshared.street.'. $_i . '"] input[name="street['. $_i . ']"]',
                                        'prefix' => 'billingAddressshared',
                                        'length'=> $streetSumLength,
                                        'message' => __($streetErrorMessage, $streetSumLength)
                                    ]
                                 ];
                            }

                             if ($_i == 0) {
                                 $_validationRule["required-entry"] = true;
                             }
                            $checkoutBillingLayoutAfter['street']['children'][$_i]['validation'] = $_validationRule;

                            $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                        }
                    }
                }
            }

            if ($cityLength = $this->_helper->getCityLength()) {

                // CITY: change template, add autocomplete property and set new validation rules.
                $checkoutShippingLayout['city']['config']['elementTmpl'] = $input_elemTmpl;
                $checkoutShippingLayout['city']['config']['autocompleteValue'] = $this->_helper->getCityAutocomplete() ? 'on' : 'nope';
                $checkoutShippingLayout['city']['validation'] = [
                    'required-entry' => true,
                    'validate-filoblu-field-length' => [
                        'target' => 'div[name="shippingAddress.city"] input[name="city"]',
                        'length'=> $cityLength,
                        'message' => __($genericErrorMessage, $cityLength)
                    ]
                ];

                $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

                if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                    // CITY: change template, add autocomplete property and set new validation rules.
                    $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                    foreach ($configuration as $paymentGroup => $groupConfig) {
                        if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['city'])) {

                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['city']['config']['elementTmpl'] = $input_elemTmpl;
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['city']['config']['autocompleteValue'] = $this->_helper->getCityAutocomplete() ? 'on' : 'nope';
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['city']['validation'] = [
                                    'required-entry' => true,
                                    'validate-filoblu-field-length' => [
                                        'target' => 'div[name="billingAddressshared.city"] input[name="city"]',
                                        'length'=> $cityLength,
                                        'message' => __($genericErrorMessage, $cityLength)
                                    ]
                                ];
                            }
                        }
                        $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                    }
              } else {
                  if (isset($checkoutBillingLayoutAfter['city'])) {

                    // CITY: change template, add autocomplete property and set new validation rules.
                    $checkoutBillingLayoutAfter['city']['config']['elementTmpl'] = $input_elemTmpl;
                    $checkoutBillingLayoutAfter['city']['config']['autocompleteValue'] = $this->_helper->getCityAutocomplete() ? 'on' : 'nope';
                    $checkoutBillingLayoutAfter['city']['validation'] = [
                        'required-entry' => true,
                        'validate-filoblu-field-length' => [
                            'target' => 'div[name="billingAddressshared.city"] input[name="city"]',
                            'length'=> $cityLength,
                            'message' => __($genericErrorMessage, $cityLength)
                        ]
                    ];

                    $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                  }
              }
          }

          if ($regionLength = $this->_helper->getRegionLength()) {

              // REGION: change template, add autocomplete property and set new validation rules.
              $checkoutShippingLayout['region_id']['component'] = 'FiloBlu_AddressFieldsLengthValidation/js/form/element/region';
              $checkoutShippingLayout['region_id']['fbconfig']['elementTmpl'] = $input_elemTmpl;
              $checkoutShippingLayout['region_id']['fbconfig']['autocompleteValue'] = $this->_helper->getRegionAutocomplete() ? 'on' : 'nope';
              $checkoutShippingLayout['region_id']['fbconfig']['validation'] = [
                   'validate-filoblu-field-length' => [
                       'target' => 'div[name="shippingAddress.region"] input[name="region"]',
                       'length'=> $regionLength,
                       'message' => __($genericErrorMessage, $regionLength)
                   ]
               ];

               $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

               if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                    // REGION: change component, template, add autocomplete property and set new validation rules.
                    $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                    foreach ($configuration as $paymentGroup => $groupConfig) {
                        if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                            if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['region_id'])) {

                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['region_id']['component'] = 'FiloBlu_AddressFieldsLengthValidation/js/form/element/region';
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['region_id']['fbconfig']['elementTmpl'] = $input_elemTmpl;
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['region_id']['fbconfig']['autocompleteValue'] = $this->_helper->getRegionAutocomplete() ? 'on' : 'nope';
                                $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['region_id']['fbconfig']['validation'] = [
                                    'validate-filoblu-field-length' => [
                                        'target' => 'div[name="billingAddressshared.region"] input[name="region"]',
                                        'length'=> $regionLength,
                                        'message' => __($genericErrorMessage, $regionLength)
                                    ]
                                ];

                                $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                            }
                        }
                    }
             } else {
                 if (isset($checkoutBillingLayoutAfter['region_id'])) {

                     // REGION: change component, template, add autocomplete property and set new validation rules.
                     $checkoutBillingLayoutAfter['region_id']['component'] = 'FiloBlu_AddressFieldsLengthValidation/js/form/element/region';
                     $checkoutBillingLayoutAfter['region_id']['fbconfig']['elementTmpl'] = $input_elemTmpl;
                     $checkoutBillingLayoutAfter['region_id']['fbconfig']['autocompleteValue'] = $this->_helper->getRegionAutocomplete() ? 'on' : 'nope';
                     $checkoutBillingLayoutAfter['region_id']['fbconfig']['validation'] = [
                                 'validate-filoblu-field-length' => [
                                     'target' => 'div[name="billingAddressshared.region"] input[name="region"]',
                                     'length'=> $regionLength,
                                     'message' => __($genericErrorMessage, $regionLength)
                                 ]
                             ];

                     $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                 }
             }
         }

         if ($postcodeLength = $this->_helper->getPostcodeLength()) {

            // POSTCODE: change template, add autocomplete property and set new validation rules.
            $checkoutShippingLayout['postcode']['config']['elementTmpl'] = $input_elemTmpl;
            $checkoutShippingLayout['postcode']['config']['autocompleteValue'] = $this->_helper->getPostcodeAutocomplete() ? 'on' : 'nope';
            $checkoutShippingLayout['postcode']['validation'] = [
                'required-entry' => true,
                'validate-filoblu-field-length' => [
                    'target' => 'div[name="shippingAddress.postcode"] input[name="postcode"]',
                    'length'=> $postcodeLength,
                    'message' => __($genericErrorMessage, $postcodeLength)
                ]
            ];

            $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['shipping-address-fieldset']['children'] = $checkoutShippingLayout;

            if ($this->_checkoutHelper->isDisplayBillingOnPaymentMethodAvailable()) {

                // POSTCODE: change template, add autocomplete property and set new validation rules.
                $configuration = $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'];
                foreach ($configuration as $paymentGroup => $groupConfig) {
                    if (isset($groupConfig['component']) && $groupConfig['component'] === 'Magento_Checkout/js/view/billing-address') {

                        if (isset($checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['postcode'])) {

                            $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['postcode']['config']['elementTmpl'] = $input_elemTmpl;
                            $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['postcode']['config']['autocompleteValue'] = $this->_helper->getPostcodeAutocomplete() ? 'on' : 'nope';
                            $checkoutBillingLayout[$paymentGroup]['children']['form-fields']['children']['postcode']['validation'] = [
                                'required-entry' => true,
                                'validate-filoblu-field-length' => [
                                    'target' => 'div[name="billingAddressshared.postcode"] input[name="postcode"]',
                                    'length'=> $postcodeLength,
                                    'message' => __($genericErrorMessage, $postcodeLength)
                                ]
                            ];

                            $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['payments-list']['children'] = $checkoutBillingLayout;
                        }
                    }
                }
            } else {
                if (isset($checkoutBillingLayoutAfter['postcode'])) {

                // POSTCODE: change template, add autocomplete property and set new validation rules.
                $checkoutBillingLayoutAfter['postcode']['config']['elementTmpl'] = $input_elemTmpl;
                $checkoutBillingLayoutAfter['postcode']['config']['autocompleteValue'] = $this->_helper->getPostcodeAutocomplete() ? 'on' : 'nope';
                $checkoutBillingLayoutAfter['postcode']['validation'] = [
                    'required-entry' => true,
                    'validate-filoblu-field-length' => [
                        'target' => 'div[name="billingAddressshared.postcode"] input[name="postcode"]',
                        'length'=> $postcodeLength,
                        'message' => __($genericErrorMessage, $postcodeLength)
                    ]
                ];

                $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children'] = $checkoutBillingLayoutAfter;
                }
             }
         }
     }

     // $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/test.log');
     // $logger = new \Zend\Log\Logger();
     // $logger->addWriter($writer);
     //
     // $logger->info($jsLayout['components']['checkout']['children']['steps']['children']['billing-step']['children']['payment']['children']['afterMethods']['children']['billing-address-form']['children']['form-fields']['children']['street']);

     return  $jsLayout;
    }
}
