define([
    'jquery',
    'mage/translate'
], function($) {
    "use strict";
    /*
     * Checkout Validations
     * The 'undefined' check if for M2 versions < 2.2 which load the validation before loading the page itself
    */
    return function(validator) {
        validator.addRule('validate-filoblu-firstname-lastname-sum-length', function(value, params, additionalParams) {
            if (typeof $(params.target).val() === 'undefined') {
                return true;
            }
            else {
                // +1 character is for the space between fields
                return ((parseInt(value.length + $(params.target).val().length) + 1 <= parseInt(params.sumLength)) && (parseInt(value.length) <= parseInt(params.length)));
            }
        }, $.mage.__('The field characters count or the sum of firstname and lastname fields exceeds maximum characters.'));

        validator.addRule('validate-filoblu-street-sum-length', function(value, params, additionalParams) {
            if (typeof $(params.target).val() === 'undefined') {
                return true;
            }
            else {
                // spaceBetweenFields: this is for counting spaces between street fields
                var spaceBetweenFields = 0;
                var streetsSum = value.length;
                $('div[name^="' + params.prefix + '.street."] input[name^="street["]:not(' + params.target + ')').each(function(index, element) {
                    spaceBetweenFields += index != 0 ? $(element).val().length > 0 ? 1 : 0 : 0;
                    streetsSum += ($(element).val().length);
                });
                streetsSum += spaceBetweenFields;
                return streetsSum <= params.length;
            }
        }, $.mage.__('The sum of the street fields exceeds maximum characters.'));

        validator.addRule('validate-filoblu-company-street-sum-length', function(value, params, additionalParams) {
            if (typeof $(params.target).val() === 'undefined') {
                return true;
            }
            else {
                // spaceBetweenFields: this is for counting spaces between street fields
                var spaceBetweenFields = ($('div[name^="' + params.prefix + '.company"] input[name="company"]').val().length) > 0 ? 1 : 0;
                var streetsSum = $('div[name^="' + params.prefix + '.company"] input[name="company"]').val().length;
                $('div[name^="' + params.prefix + '.street."] input[name^="street["]').each(function(index, element) {
                    spaceBetweenFields += index != 0 ? $(element).val().length > 0 ? 1 : 0 : 0;
                    streetsSum += ($(element).val().length);
                });
                streetsSum += spaceBetweenFields;
                return streetsSum <= params.length;
            }
        }, $.mage.__('The sum of the street fields (including company) exceeds maximum characters.'));

        validator.addRule('validate-filoblu-field-length', function(value, params, additionalParams) {
            if (typeof ($(params.target).val()) === 'undefined') {
                return true;
            }
            else {
                return parseInt($(params.target).val().length) <= parseInt(params.length);
            }
        }, $.mage.__('The field exceeds maximum characters.'));

        return validator;
    };
});
